@@ -322,27 +322,61 @@ class StructuredWave(AcousticField):
         """
         Set up source for both 2D and 3D structured waves.
         """
+        active_list = np.array([int(char) for char in ''.join(f"{int(self.pattern.activeList[i:i+2], 16):08b}" for i in range(0, len(self.pattern.activeList), 2))])
         element_width_grid_points = int(round(self.params['element_width'] / dx))
 
         if source.p_mask.ndim == 2:
-            # --- 2D ---
+            element_width_grid_points = int(round(self.params['element_width'] / dx))
             total_elements_width = self.params['num_elements'] * element_width_grid_points
-            current_position = (Nx - total_elements_width) // 2
-            active_grid_indices = []
 
+            # Vérifier que les éléments rentrent dans le grid
+            if total_elements_width > Nx:
+                raise ValueError(f"La largeur totale des éléments ({total_elements_width}) dépasse Nx ({Nx}).")
+
+            remaining_space = Nx - total_elements_width
+            if remaining_space < 0:
+                raise ValueError(f"Pas assez d'espace pour placer les éléments: total_elements_width ({total_elements_width}) > Nx ({Nx}).")
+
+            spacing = remaining_space // (self.params['num_elements'] + 1)
+            center_index = np.argmin(np.abs(np.linspace(self.params['Xrange'][0], self.params['Xrange'][1], Nx)))
+
+            activeListGrid = np.zeros(total_elements_width, dtype=int)
+            current_position = center_index - (total_elements_width + (self.params['num_elements'] - 1) * spacing) // 2
+
+            # Placement des éléments actifs
             for i in range(self.params['num_elements']):
-                x_start = current_position
-                x_end = current_position + element_width_grid_points
-                source.p_mask[x_start:x_end, 0] = 1
-                active_grid_indices.extend(range(x_start, x_end))
-                current_position += element_width_grid_points
+                if active_list[i] == 1:
+                    x_pos = max(0, current_position)  # Éviter les indices négatifs
+                    x_end = x_pos + element_width_grid_points
+                    if x_end > Nx:
+                        x_end = Nx  # Limiter à Nx
+                    source.p_mask[x_pos:x_end,0] = 1
+
+                    start_idx = i * element_width_grid_points
+                    end_idx = start_idx + element_width_grid_points
+                    if end_idx > total_elements_width:
+                        end_idx = total_elements_width
+                    activeListGrid[start_idx:end_idx] = 1
+
+                current_position += element_width_grid_points + spacing
+
+            # Chargement des signaux retardés
+            if factorT != 1:
+                delayedSignal = self._apply_delay(dx=dx)
+            else:
+                delayedSignal = self.delayedSignal
+
+            # Vérification de la taille de delayedSignal
+            num_active_elements = np.sum(activeListGrid == 1)
+            if delayedSignal.shape[0] < num_active_elements:
+                raise ValueError(f"delayedSignal a une taille insuffisante: {delayedSignal.shape[0]} < {num_active_elements}.")
+
+            # Assigner source.p
+            source.p =  float(self.params['voltage']) * float(self.params['sensitivity']) * delayedSignal[activeListGrid == 1, :]
 
-            delayed_signals = self._apply_delay(dx=dx) if factorT != 1 else self.delayedSignal
-            source.p = delayed_signals[active_grid_indices, :]
 
         elif source.p_mask.ndim == 3:
             # --- 3D ---
-            active_list = np.array([int(char) for char in ''.join(f"{int(self.pattern.activeList[i:i+2], 16):08b}" for i in range(0, len(self.pattern.activeList), 2))])
             center_index_x = Nx // 2
             center_index_y = self.params['Ny'] // 2
             spacing = (Nx - self.params['num_elements'] * element_width_grid_points) // (self.params['num_elements'] + 1)
@@ -355,4 +389,4 @@ class StructuredWave(AcousticField):
                 current_position += element_width_grid_points + spacing
 
             delayed_signals = self._apply_delay()
-            source.p = delayed_signals.T
\ No newline at end of file
+            source.p =  float(self.params['voltage']) * float(self.params['sensitivity']) * delayed_signals.T
\ No newline at end of file
