@@ -13,6 +13,7 @@ from datetime import datetime
 import matplotlib.pyplot as plt
 import matplotlib.animation as animation
 import matplotlib as mpl
+import copy
 
 class Experiment(ABC):
     def __init__(self, params, acousticType=WaveType.StructuredWave, formatSave=FormatSave.HDR_IMG):
@@ -31,6 +32,10 @@ class Experiment(ABC):
         if type(self.params) != Params:
             raise TypeError("params must be an instance of the Params class")
 
+    def copy(self):
+        """Retourne une copie profonde de l'objet."""
+        return copy.deepcopy(self)
+    
     def generatePhantom(self):
         """
         Generate the phantom for the experiment.
@@ -86,26 +91,31 @@ class Experiment(ABC):
             raise ValueError("AO signal without tumor is not generated. Please generate it first.")
 
         if withTumor:
-            AOsignal = self.AOsignal_withTumor
+            AOsignals = self.AOsignal_withTumor
         else:
-            AOsignal = self.AOsignal_withoutTumor
-
-        if noiseType.lower() == 'gaussian':
-            noise = np.random.normal(0, noiseLvl, AOsignal.shape)
-            noisy_signal = AOsignal + noise
-        elif noiseType.lower() == 'poisson':
-            # Pour le bruit de Poisson, on utilise souvent un facteur multiplicatif
-            # car le bruit de Poisson est proportionnel à la racine carrée du signal.
-            # Ici, on multiplie le signal par un facteur aléatoire centré autour de 1.
-            noise = np.random.poisson(noiseLvl * np.abs(AOsignal)) / (noiseLvl * np.abs(AOsignal).max())
-            noisy_signal = AOsignal * noise
-        else:
-            raise ValueError("noiseType must be either 'gaussian' or 'poisson'.")
+            AOsignals = self.AOsignal_withoutTumor
+        
+        noiseSignals = np.zeros_like(AOsignals)
+        for i in trange(AOsignals.shape[1], desc=f"Adding {noiseType} noise to AO signal {'with' if withTumor else 'without'} tumor"):
+            AOsignal = AOsignals[:, i]
+            if noiseType.lower() == 'gaussian':
+                noise = np.random.normal(0, noiseLvl*np.max(AOsignal), AOsignal.shape)
+                noisy_signal = AOsignal + noise
+            elif noiseType.lower() == 'poisson':
+                # Pour le bruit de Poisson, on utilise souvent un facteur multiplicatif
+                # car le bruit de Poisson est proportionnel à la racine carrée du signal.
+                # Ici, on multiplie le signal par un facteur aléatoire centré autour de 1.
+                noise = np.random.poisson(noiseLvl * np.abs(AOsignal)) / (noiseLvl * np.abs(AOsignal).max())
+                noisy_signal = AOsignal * noise
+            else:
+                raise ValueError("noiseType must be either 'gaussian' or 'poisson'.")
+            noisy_signal = np.clip(noisy_signal, a_min=0, a_max=None)  # Assurer que le signal reste non négatif
+            noiseSignals[:, i] = noisy_signal
 
         if withTumor:
-            self.AOsignal_withTumor = noisy_signal
+            self.AOsignal_withTumor = noiseSignals
         else:
-            self.AOsignal_withoutTumor = noisy_signal
+            self.AOsignal_withoutTumor = noiseSignals
 
 
     def reduceDims(self, mode='avg'):
